#include "lpc210x.h"

void SystemInit(void)  {   
	// Setting Multiplier and Divider values
  	PLLCFG=PSEL | MSEL;
	PLLFEED=PLL_FEED1;
	PLLFEED=PLL_FEED2;
	
	// Enabling the PLL
	PLLCON=PLLE;
	PLLFEED=PLL_FEED1;
	PLLFEED=PLL_FEED2;
  
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLLSTAT_PLOCK)) ;
  
	// Connect and enable the PLL as the clock source
	PLLCON=PLLC|PLLE;
	PLLFEED=PLL_FEED1;
	PLLFEED=PLL_FEED2;
  
	// Make sure the Memory Access Module is turned off
	MAMCR=MAMMODE_DISABLE;
	// Change the timing to 3 wait cycles
	#if CCLK < 20000000
	MAMTIM=1;
	#elseif CCLK < 40000000
	MAMTIM=2;
	#else
	MAMTIM=3;
	#endif
	// Enable the MAM
	MAMCR=MAMMODE_ENABLE;
	
	// Setting peripheral Clock (pclk) to System Clock (cclk)
	VPBDIV=VPBDIV_ONE;
	
	MEMMAP = MAP;
}

void enableIRQ(void) {
	asm volatile (
		"mrs	r0, cpsr\n"
		"and	r0, r0, #~0x00000080 \n"
		"msr	cpsr, r0\n"
	);
}

void disableIRQ(void) {
	asm volatile (
		"mrs	r0, cpsr\n"
		"orr	r0, r0, #0x00000080 \n"
		"msr	cpsr, r0\n"
	);
}

void StartRTC(void) {
	//calculate a new divider first
	RTC_PREINT = (CCLK / 32768)-1; // int(pclk / 32768)-1
	RTC_PREFRAC = CCLK - ((RTC_PREINT+1) * 32768);
	
	RTC_CCR = CLKEN;
}

void StopRTC(void) {
	RTC_CCR = 0;
}

void SetTime(u8 hours, u8 minutes, u8 seconds) {
	RTC_CCR |= CTCRST; //probably not needed
	RTC_HOUR = hours; 
	RTC_MIN = minutes;
	RTC_SEC = seconds;
	RTC_CCR &= ~CTCRST;
}

void SetDate(u8 day, u8 month, u16 year) {
	RTC_DOM = day; 
	RTC_MONTH = month;
	RTC_YEAR = year;
}
