#include "lpc210x.h"

#include "SOUND.H"

#define VICVectCntl0_ENABLE (1<<5)

#define VIC_Channel_Timer0  4

#define TxTCR_COUNTER_ENABLE (1<<0)
#define TxTCR_COUNTER_RESET  (1<<1)
#define TxMCR_INT_ON_MR0     (1<<0)
#define TxMCR_RESET_ON_MR0   (1<<1)
#define TxIR_MR0_FLAG        (1<<0)

void init_timer (void);
void __attribute__ ((interrupt("IRQ"))) tc0_cmp(void) ;

unsigned long	songPos=0;

/* Timer0 Compare-Match Interrupt Handler (ISR) */
void tc0_cmp(void)  {
	//setup PWM timing
	PWM_MR0 = 256;				// set MR0 (rate)
	PWM_MR4 = sound[songPos];	// set MR4

	//activate updates
	PWM_TCR = 9;		//mode
	PWM_LER |= 0x10;	//activate update
	
	songPos++;
	if (songPos >= SOUND_LEN)
		songPos = 0;
		
	T0IR = TxIR_MR0_FLAG;	// Clear interrupt flag by writing 1 to Bit 0
	VICVectAddr = 0;		// Acknowledge Interrupt (rough?)
}

/* Setup Timer0 Compare-Match Interrupt         */
/* no prescaler timer runs at cclk = FOSC*PLL_M */
void init_timer (void) {
	T0MR0 = (CCLK/(8000))-1;     // 8000 Hz (-1 reset "tick")
	T0MCR = TxMCR_INT_ON_MR0 | TxMCR_RESET_ON_MR0; 	// Interrupt and Reset on MR0
	T0TCR = TxTCR_COUNTER_ENABLE;            // Timer0 Enable

	VICVectAddr0 = (unsigned long)tc0_cmp;   // set interrupt vector in 0
	VICVectCntl0 = VICVectCntl0_ENABLE | VIC_Channel_Timer0; // use it for Timer 0 Interrupt:
	VICIntEnable = (1<<VIC_Channel_Timer0);    // Enable Timer0 Interrupt
}


int main(void) {	
	SystemInit();
	
	enableIRQ();

	//no prescaler, maximal frequency for best possible sound quality
	PWM_PR = 0;
	PINSEL0 |= 0x00020005;	//enable
	PWM_MCR |= 2;			//reset on MR0
	PWM_PCR |= 0x1000;		//PWM5 output
	init_timer();
		
	while (1);
	
	return 0;
}






